/* SPDX-License-Identifier: GPL-2.0 */
/* Copyright(c) 2009-2014  Realtek Corporation.*/

#ifndef __RTL92E_TABLE__H_
#define __RTL92E_TABLE__H_

#include <linux/types.h>
#define RTL8192EE_PHY_REG_ARRAY_LEN	448
extern u32 RTL8192EE_PHY_REG_ARRAY[];
#define RTL8192EE_PHY_REG_ARRAY_PG_LEN	168
extern u32 RTL8192EE_PHY_REG_ARRAY_PG[];
#define	RTL8192EE_RADIOA_ARRAY_LEN	238
extern u32 RTL8192EE_RADIOA_ARRAY[];
#define	RTL8192EE_RADIOB_ARRAY_LEN	198
extern u32 RTL8192EE_RADIOB_ARRAY[];
#define RTL8192EE_MAC_ARRAY_LEN		202
extern u32 RTL8192EE_MAC_ARRAY[];
#define RTL8192EE_AGC_TAB_ARRAY_LEN	532
extern u32 RTL8192EE_AGC_TAB_ARRAY[];

#endif
