/*
 * Automatically generated by asn1_compiler.  Do not edit
 *
 * ASN.1 parser for pkcs7
 */
#include <linux/asn1_decoder.h>

extern const struct asn1_decoder pkcs7_decoder;

extern int pkcs7_check_content_type(void *, size_t, unsigned char, const void *, size_t);
extern int pkcs7_extract_cert(void *, size_t, unsigned char, const void *, size_t);
extern int pkcs7_note_OID(void *, size_t, unsigned char, const void *, size_t);
extern int pkcs7_note_certificate_list(void *, size_t, unsigned char, const void *, size_t);
extern int pkcs7_note_content(void *, size_t, unsigned char, const void *, size_t);
extern int pkcs7_note_data(void *, size_t, unsigned char, const void *, size_t);
extern int pkcs7_note_signed_info(void *, size_t, unsigned char, const void *, size_t);
extern int pkcs7_note_signeddata_version(void *, size_t, unsigned char, const void *, size_t);
extern int pkcs7_note_signerinfo_version(void *, size_t, unsigned char, const void *, size_t);
extern int pkcs7_sig_note_authenticated_attr(void *, size_t, unsigned char, const void *, size_t);
extern int pkcs7_sig_note_digest_algo(void *, size_t, unsigned char, const void *, size_t);
extern int pkcs7_sig_note_issuer(void *, size_t, unsigned char, const void *, size_t);
extern int pkcs7_sig_note_pkey_algo(void *, size_t, unsigned char, const void *, size_t);
extern int pkcs7_sig_note_serial(void *, size_t, unsigned char, const void *, size_t);
extern int pkcs7_sig_note_set_of_authattrs(void *, size_t, unsigned char, const void *, size_t);
extern int pkcs7_sig_note_signature(void *, size_t, unsigned char, const void *, size_t);
extern int pkcs7_sig_note_skid(void *, size_t, unsigned char, const void *, size_t);

