/*
 * Automatically generated by asn1_compiler.  Do not edit
 *
 * ASN.1 parser for x509_akid
 */
#include <linux/asn1_decoder.h>

extern const struct asn1_decoder x509_akid_decoder;

extern int x509_akid_note_kid(void *, size_t, unsigned char, const void *, size_t);
extern int x509_akid_note_name(void *, size_t, unsigned char, const void *, size_t);
extern int x509_akid_note_serial(void *, size_t, unsigned char, const void *, size_t);
extern int x509_extract_name_segment(void *, size_t, unsigned char, const void *, size_t);
extern int x509_note_OID(void *, size_t, unsigned char, const void *, size_t);

