/* Automatically created during backport process */
#ifndef CPTCFG_BPAUTO_REFCOUNT
#include_next <linux/refcount.h>
#else
#undef refcount_add_not_zero_checked
#define refcount_add_not_zero_checked LINUX_BACKPORT(refcount_add_not_zero_checked)
#undef refcount_add_checked
#define refcount_add_checked LINUX_BACKPORT(refcount_add_checked)
#undef refcount_inc_not_zero_checked
#define refcount_inc_not_zero_checked LINUX_BACKPORT(refcount_inc_not_zero_checked)
#undef refcount_inc_checked
#define refcount_inc_checked LINUX_BACKPORT(refcount_inc_checked)
#undef refcount_sub_and_test_checked
#define refcount_sub_and_test_checked LINUX_BACKPORT(refcount_sub_and_test_checked)
#undef refcount_dec_and_test_checked
#define refcount_dec_and_test_checked LINUX_BACKPORT(refcount_dec_and_test_checked)
#undef refcount_dec_checked
#define refcount_dec_checked LINUX_BACKPORT(refcount_dec_checked)
#undef refcount_dec_if_one
#define refcount_dec_if_one LINUX_BACKPORT(refcount_dec_if_one)
#undef refcount_dec_not_one
#define refcount_dec_not_one LINUX_BACKPORT(refcount_dec_not_one)
#undef refcount_dec_and_mutex_lock
#define refcount_dec_and_mutex_lock LINUX_BACKPORT(refcount_dec_and_mutex_lock)
#undef refcount_dec_and_lock
#define refcount_dec_and_lock LINUX_BACKPORT(refcount_dec_and_lock)
#undef refcount_dec_and_lock_irqsave
#define refcount_dec_and_lock_irqsave LINUX_BACKPORT(refcount_dec_and_lock_irqsave)
#include <linux/backport-refcount.h>
#endif /* CPTCFG_BPAUTO_REFCOUNT */
