/* SPDX-License-Identifier: GPL-2.0 */
/* Copyright(c) 2009-2012  Realtek Corporation.*/

#ifndef __RTL8723E_BTC_H__
#define __RTL8723E_BTC_H__

#include "../wifi.h"
#include "hal_bt_coexist.h"

struct bt_coexist_c2h_info {
	u8 no_parse_c2h;
	u8 has_c2h;
};

#endif
