/*
 * Automatically generated by asn1_compiler.  Do not edit
 *
 * ASN.1 parser for rsapubkey
 */
#include <linux/asn1_decoder.h>

extern const struct asn1_decoder rsapubkey_decoder;

#define rsa_get_e LINUX_BACKPORT(rsa_get_e)
#define rsa_get_n LINUX_BACKPORT(rsa_get_n)

extern int rsa_get_e(void *, size_t, unsigned char, const void *, size_t);
extern int rsa_get_n(void *, size_t, unsigned char, const void *, size_t);

