#ifndef __MBEDTLS_CONFIG_H
#define __MBEDTLS_CONFIG_H
#define MBEDTLS_RSA_C
#define MBEDTLS_PKCS1_V15
#define MBEDTLS_MD_C
#define __OpenBSD__
#define MBEDTLS_PLATFORM_C
#define MBEDTLS_BIGNUM_C
#define MBEDTLS_OID_C
#define MBEDTLS_ASN1_PARSE_C
#define MBEDTLS_NO_UDBL_DIVISION
#define MBEDTLS_SHA256_C
#include <linux/types.h>
#include "platform.h"
#endif /* __MBEDTLS_CONFIG_H */
